AS {{ cfg.rs_as }}
router-id {{ cfg.router_id }}

fib-update no
{% if not "logging" is used_local_file %}
log updates
{% else %}
{{ "logging"|include_local_file -}}
{% endif %}

nexthop qualify via default

{% if cfg.path_hiding and "6.9"|target_version_ge %}
rde evaluate all
{% endif %}

INTCOMM_PREF_OK_ROA="soo 65535:1"
INTCOMM_ROUTE_OK_WL="soo 65535:2"
INTCOMM_PREF_OK_ARINDB="soo 65535:3"
INTCOMM_PREF_OK_REGISTROBRDB="soo 65535:12"

INTCOMM_ORIGIN_OK="soo 65535:4"
INTCOMM_ORIGIN_KO="soo 65535:5"
INTCOMM_PREFIX_OK="soo 65535:6"
INTCOMM_PREFIX_KO="soo 65535:7"
INTCOMM_IRR_REJECT="soo 65535:8"

INTCOMM_RPKI_UNKNOWN="soo 65535:9"
INTCOMM_RPKI_INVALID="soo 65535:10"
INTCOMM_RPKI_VALID="soo 65535:11"

INTCOMM_NO_EXPORT="soo 65535:65281"
INTCOMM_NO_ADVERTISE="soo 65535:65282"
